@@ -449,10 +449,30 @@ yelp_view_finalize (GObject *object)
     G_OBJECT_CLASS (yelp_view_parent_class)->finalize (object);
 }
 
+static gboolean
+yelp_view_button_press_event (GtkWidget      *widget,
+                              GdkEventButton *event)
+{
+    /* Handle typical back/forward mouse buttons. */
+    if (event->button == 8) {
+        webkit_web_view_go_back (WEBKIT_WEB_VIEW (widget));
+        return TRUE;
+    }
+
+    if (event->button == 9) {
+        webkit_web_view_go_forward (WEBKIT_WEB_VIEW (widget));
+        return TRUE;
+    }
+
+     /* Let parent class handle this. */
+     return GTK_WIDGET_CLASS (yelp_view_parent_class)->button_press_event (widget, event);
+}
+
 static void
 yelp_view_class_init (YelpViewClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
     YelpSettings *settings = yelp_settings_get_default ();
 
     nautilus_sendto = g_find_program_in_path ("nautilus-sendto");
@@ -562,6 +582,8 @@ yelp_view_class_init (YelpViewClass *klass)
                                                           NULL,
                                                           G_PARAM_READABLE |
                                                           G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+    widget_class->button_press_event = yelp_view_button_press_event;
 }
 
 static void
@@ -758,17 +780,19 @@ document_callback (YelpDocument       *document,
 
     contents = yelp_document_read_contents (document, data->page_id);
 
-    content_length = strlen (contents);
+    if (contents) {
+        content_length = strlen (contents);
 
-    stream = g_memory_input_stream_new_from_data (g_strdup (contents), content_length, g_free);
-    yelp_document_finish_read (document, contents);
+        stream = g_memory_input_stream_new_from_data (g_strdup (contents), content_length, g_free);
+        yelp_document_finish_read (document, contents);
 
-    webkit_uri_scheme_request_finish (data->request,
-                                      stream,
-                                      content_length,
-                                      mime_type);
-    g_free (mime_type);
-    g_object_unref (stream);
+        webkit_uri_scheme_request_finish (data->request,
+                                          stream,
+                                          content_length,
+                                          mime_type);
+        g_free (mime_type);
+        g_object_unref (stream);
+    }
 }
 
 static void
@@ -1825,11 +1849,12 @@ view_clear_load (YelpView *view)
 static gchar*
 fix_docbook_uri (YelpUri *docbook_uri, YelpDocument* document)
 {
-    SoupURI *soup_uri;
+    GUri *uri;
     gchar *retval, *canonical;
+    const gchar *fragment;
 
     canonical = yelp_uri_get_canonical_uri (docbook_uri);
-    soup_uri = soup_uri_new (canonical);
+    uri = g_uri_parse (canonical, G_URI_FLAGS_ENCODED, NULL);
     g_free (canonical);
 
     /* We don't have actual page and frag IDs for DocBook. We just map IDs
@@ -1837,19 +1862,32 @@ fix_docbook_uri (YelpUri *docbook_uri, YelpDocument* document)
        If someid is really the page ID, we just drop the frag reference.
        Otherwise, normal page views scroll past the link trail.
     */
-    if (soup_uri->fragment && YELP_IS_DOCBOOK_DOCUMENT (document)) {
+    fragment = g_uri_get_fragment (uri);
+    if (fragment && YELP_IS_DOCBOOK_DOCUMENT (document)) {
         gchar *page_id = yelp_uri_get_page_id (docbook_uri);
         gchar *real_id = yelp_document_get_page_id (document, page_id);
 
-        if (g_str_equal (real_id, soup_uri->fragment))
-            soup_uri_set_fragment (soup_uri, NULL);
+        if (g_str_equal (real_id, fragment)) {
+            GUri *modified;
+
+            modified = g_uri_build (g_uri_get_flags (uri),
+                                    g_uri_get_scheme (uri),
+                                    g_uri_get_userinfo (uri),
+                                    g_uri_get_host (uri),
+                                    g_uri_get_port (uri),
+                                    g_uri_get_path (uri),
+                                    g_uri_get_query (uri),
+                                    NULL);
+            g_uri_unref (uri);
+            uri = modified;
+        }
 
         g_free (real_id);
         g_free (page_id);
     }
 
-    retval = soup_uri_to_string (soup_uri, FALSE);
-    soup_uri_free (soup_uri);
+    retval = g_uri_to_string (uri);
+    g_uri_unref (uri);
 
     return retval;
 }
@@ -1904,42 +1942,13 @@ view_load_page (YelpView *view)
 #define FORMAT_ERRORPAGE \
     "<html><head>" \
     "<style type='text/css'>" \
-    "body {" \
-    " margin: 1em;" \
-    " color: %s;" \
-    " background-color: %s;" \
-    " }\n" \
-    "p { margin: 1em 0 0 0; }\n" \
-    "div.note {" \
-    " padding: 6px;" \
-    " border-color: %s;" \
-    " border-top: solid 1px;" \
-    " border-bottom: solid 1px;" \
-    " background-color: %s;" \
-    " }\n" \
-    "div.note div.inner {" \
-    " margin: 0; padding: 0;" \
-    " background-image: url(%s);" \
-    " background-position: %s top;" \
-    " background-repeat: no-repeat;" \
-    " min-height: %ipx;" \
-    " }\n" \
-    "div.note div.contents {" \
-    " margin-%s: %ipx;" \
-    " }\n" \
-    "div.note div.title {" \
-    " margin-%s: %ipx;" \
-    " margin-bottom: 0.2em;" \
-    " font-weight: bold;" \
-    " color: %s;" \
-    " }\n" \
-    "a { color: %s; text-decoration: none; }\n" \
-    "</style>" \
-    "</head><body>" \
-    "<div class='note'><div class='inner'>" \
-    "%s<div class='contents'>%s%s</div>" \
-    "</div></div>" \
-    "</body></html>"
+    " body { color: %s; background-color: %s; } " \
+    " div.title { font-size: 2em; margin: 1rem 0 0 0; } " \
+    " p { margin: 1em 0 0 0; } " \
+    " .container { max-width: 800px; margin: 0 auto; text-align: center; } " \
+    "</style></head><body>" \
+    "<div class='container'>" \
+    "<div><img src='%s' height='256'></div>%s%s%s</div></body></html>"
 
 static void
 view_show_error_page (YelpView *view,
@@ -1947,12 +1956,12 @@ view_show_error_page (YelpView *view,
 {
     YelpViewPrivate *priv = yelp_view_get_instance_private (view);
     YelpSettings *settings = yelp_settings_get_default ();
+    GtkIconTheme *icontheme;
+    GtkIconInfo *icon;
     gchar *page, *title = NULL, *title_m, *content_beg, *content_end;
-    gchar *textcolor, *bgcolor, *noteborder, *notebg, *titlecolor, *noteicon, *linkcolor;
-    gint iconsize;
+    gchar *textcolor, *bgcolor;
     GParamSpec *spec;
     gboolean doc404 = FALSE;
-    const gchar *left = (gtk_widget_get_direction ((GtkWidget *) view) == GTK_TEXT_DIR_RTL) ? "right" : "left";
 
     if (priv->uri && yelp_uri_get_document_type (priv->uri) == YELP_URI_DOCUMENT_TYPE_NOT_FOUND)
         doc404 = TRUE;
@@ -1997,18 +2006,17 @@ view_show_error_page (YelpView *view,
 
     textcolor = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_TEXT);
     bgcolor = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_BASE);
-    noteborder = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_RED_BORDER);
-    notebg = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_YELLOW_BASE);
-    titlecolor = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_TEXT_LIGHT);
-    linkcolor = yelp_settings_get_color (settings, YELP_SETTINGS_COLOR_LINK);
-    noteicon = yelp_settings_get_icon (settings, YELP_SETTINGS_ICON_WARNING);
-    iconsize = yelp_settings_get_icon_size (settings) + 6;
-
+    g_object_get (settings, "gtk-icon-theme", &icontheme, NULL);
+    icon = gtk_icon_theme_lookup_icon (icontheme, "computer-fail-symbolic", 256, 0);
     page = g_strdup_printf (FORMAT_ERRORPAGE,
-                            textcolor, bgcolor, noteborder, notebg, noteicon,
-                            left, iconsize, left, iconsize, left, iconsize,
-                            titlecolor, linkcolor, title_m, content_beg,
+                            textcolor, bgcolor,
+                            gtk_icon_info_get_filename (icon),
+                            title_m,
+                            content_beg,
                             (content_end != NULL) ? content_end : "");
+    g_object_unref (icon);
+    g_free (textcolor);
+    g_free (bgcolor);
 
     g_object_set (view, "state", YELP_VIEW_STATE_ERROR, NULL);
 
